/*
  only_WSPR2fixgps_05.01.20_i2c

  LCD_i2c     LCD_PCB Pinout(Standart LCD 16x2)
  VSS --->    1
  VSS --->    2
  SDA --->    6
  SCL --->    4
  ------------------------------------------------------------
*/
// include the library code:
#include <LiquidCrystal_I2C.h> 
#include <EEPROM.h>

struct CalFact {
  union {
    long value;
    struct {
      unsigned char b1;
      unsigned char b2;
      unsigned char b3;
      unsigned char b4;
    }
    __attribute__((packed));
  }
  __attribute__((packed));
}
__attribute__((packed));
struct CalFact CalFactor;


//__________________________________________________________________________________________________
// ENTER WSPR DATA:
char call[7] = "RA4FHE";
char locator[5] = "LO23"; // Use 4 character locator e.g. "EM64"
byte power = 17; // Min = 0 dBm, Max = 43 dBm, steps 0,3,7,10,13,17,20,23,27,30,33,37,40,43
//__________________________________________________________________________________________________

//__________________________________________________________________________________________________
// LOAD BAND FREQUENCY DATA:
unsigned long band[10] = {
  3570100,  // timeslot 0  00,20,40 minutes after hour
  7040100,  // timeslot 1  02,22,42 minutes after hour
  10140200, // timeslot 2  04,24.44 minutes after hour
  14097100, // timeslot 3  06,26,46 minutes after hour
  18106100, // timeslot 4  08,28,48 minutes after hour
  21096100, // timeslot 5  10,30,50 minutes after hour
  24926100, // timeslot 6  12,32,52 minutes after hour
  28126100, // timeslot 7  14,34,54 minutes after hour
  1838100,  // timeslot 8  16,36,56 minutes after hour
  50294500  // timeslot 9  18,38,58 minutes after hour
};

//__________________________________________________________________________________________________
// LOAD TRANSMIT TIME SLOT DATA: ( 0=idle, 1=transmit WSPR )
const byte TransmitFlag[10] = {
  1, // timeslot 0
  1, // timeslot 1
  1, // timeslot 2
  1, // timeslot 3
  1, // timeslot 4
  1, // timeslot 5
  1, // timeslot 6
  1, // timeslot 7
  0, // timeslot 8
  0  // timeslot 9
};

// AD9850 AD9851 selection
byte w0 = 0;                     // 0 = AD9850, 1 = AD9851
unsigned long fCLK = 125000000;  // Enter clock frequency

//__________________________________________________________________________________________________

const char SyncVec[162] = {
  1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0,
  1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0,
  0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0, 0,
  0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0
};


#define InhibitButton  2
#define CalSetButton   2
#define CalUpButton    1
#define CalDwnButton   3
#define TimeSetButton  3
#define MinSetButton   2
#define HourSetButton  1

#define IndicesCount 13
// initialize the library with the numbers of the interface pins
LiquidCrystal_I2C lcd(0x27, 20, 4);

// configure variables
int GPSpin = 0;             // RX PIN
int txPin = 13;             // TX (HIGH on transmit)
int BandBCD0pin = 7;
int BandBCD1pin = 8;
int BandBCD2pin = 9;
int BandBCD3pin = 10;
int byteGPS = -1;
char buffer[300] = "";
char StartCommand[7] = "$GNGGA",StartCommand2[7] = "$GNRMC";
int IndiceCount = 0, StartCount = 0, counter = 0;
int indices[IndicesCount];
int second = 0, minute = 0, hour = 0;
int Lat10, Lat1, NS, Lon100, Lon10, Lon1, EW, validGPSflag;
byte pinState = 0;
int mSecTimer, mSecTimer2, sat1, sat10;
unsigned int OffsetFreq[4];
int val = 0;     // variable for reading the pin status
byte data = 6; //DATA
byte clock = 5; //CLOCK
byte load = 4; //LOAD
byte count = 0, start = 0;
unsigned long mask = 1; // bitmask
byte temp = 1;
int inc = 3;
unsigned long FreqWord, TempWord, TempFreq, SineDelay;
unsigned long TenMHz = 10000000;
//long CalFactor;
char buf[10];
volatile byte bb, i, j, ii, value, timeslot;
byte symbol[162];
byte c[11];                // encoded message
byte sym[170];             // symbol table 162
byte symt[170];            // symbol table temp
byte RXflag = 1;  // GPS receiver control 0 = disable, 1 = enable
int MsgLength;
volatile byte InhibitFlag = 0, GPSinhibitFlag = 1; // 1 will inhibit transmitter
unsigned long n1;    // encoded callsign
unsigned long m1;    // encodes locator
char cnt1;
unsigned char cc1;
float pi = 3.14, wavecount = 0;
double wave;
byte flagUpdateTime = 0, flagUpdateTx = 0;

ISR(TIMER1_COMPA_vect) {
  second++ ;
  if (second == 60) {
    minute++ ;
    second = 0 ;
  }
  if (minute == 60) {
    hour++;
    minute = 0 ;
  }
  if (hour == 24) {
    hour = 0 ;
  }
  flagUpdateTime = 1;
  if (InhibitFlag == 0)
  {
    if (bitRead(minute, 0) == 0 && second == 0)
    {
      if (minute < 20) {
        timeslot = minute / 2;
      }
      else {
        if (minute < 40) {
          timeslot = (minute - 20) / 2;
        }
        else {
          timeslot = (minute - 40) / 2;
        }
      }
      flagUpdateTx = 1;
    }
  }
};

//******************************************************************
//Timer2 Overflow Interrupt Vector, called every mSec to increment
//the mSecTimer2 used for WSPR transmit timing.
//Timer2 is also used for QRSS pattern generation

ISR(TIMER2_COMPA_vect) {

    mSecTimer2++;
    if (mSecTimer2 > 681) {
      mSecTimer2 = 0;
      if (bb < 3) { // Begin 2 second delay - actually 0.682mSec * 3
        TempWord = FreqWord;
        TransmitSymbol();
        bb++;
      }
      else
      {
        // Begin 162 WSPR symbol transmission
        if (count < 162)
        {
          TempWord = FreqWord + OffsetFreq[sym[count]];
          TransmitSymbol();
          count++;             //Increments the interrupt counter
        }
        else
        {
          TIMSK2 = 0;   // Disable WSPR timer
          digitalWrite(txPin, LOW); // External transmit control OFF
          TempWord = 0;          // Turn off transmitter
          TransmitSymbol();
          GpsEnable();          //Turn GPS receiver back on
        }
      }
    }
  };


//******************************************************************
//******************************************************************

void GpsEnable()
{
    while (Serial.read() != -1);
    RXflag = 1;          //Turn GPS receiver back on
}

void setup()
{
//  MySW.start();

  //Set up Timer2 to fire every mSec (WSPR timer)
  TIMSK2 = 0;        //Disable timer during setup
  TCCR2A = 2;        //CTC mode
  TCCR2B = 4;        //Timer Prescaler set to 64
  OCR2A = 247;       // Timer set for 1 mSec with correction factor

  //Set up Timer1A to fire every second (master clock)
  TCCR1B = 0;        //Disable timer during setup
  TIMSK1 = 2;        //Timer1 Interrupt enable
  TCCR1A = 0;        //Normal port operation, Wave Gen Mode normal
  TCCR1B = 12;       //Timer prescaler to 256 - CTC mode
  OCR1A = 62377;     //Timer set for 1000 mSec using correction factor
  // 62500 is nominal for 1000 mSec. Decrease variable to increase clock speed

  //Load offset values
  OffsetFreq[0] = 0;
  OffsetFreq[1] = 1.43 * pow(2, 32) / fCLK;
  OffsetFreq[2] = 2.93 * pow(2, 32) / fCLK;
  OffsetFreq[3] = 4.39 * pow(2, 32) / fCLK;


  // set up the LCD for 16 columns and 2 rows
    lcd.init();          
    lcd.backlight();
  
  lcd.setCursor(0, 0);
  lcd.print("Loading...");

  //Set up GPS pin
  pinMode(GPSpin, INPUT);
  digitalWrite(GPSpin, HIGH); // internal pull-up enabled

  // Set up TX pin to output
  pinMode(txPin, OUTPUT);

  //Set up band select pins
  pinMode(BandBCD0pin, OUTPUT);
  pinMode(BandBCD1pin, OUTPUT);
  pinMode(BandBCD2pin, OUTPUT);
  pinMode(BandBCD3pin, OUTPUT);

   // Set up transmit inhibit interrupt
  pinMode(InhibitButton, INPUT);
  digitalWrite(InhibitButton, HIGH); // internal pull-up enabled

  // Set up calibration pins
  pinMode(CalSetButton, INPUT);    // declare pushbutton as input
  digitalWrite(CalSetButton, HIGH); // internal pull-up enabled
  pinMode(CalUpButton, INPUT);    // declare pushbutton as input
  digitalWrite(CalUpButton, HIGH); // internal pull-up enabled
  pinMode(CalDwnButton, INPUT);    // declare pushbutton as input
  digitalWrite(CalDwnButton, HIGH); // internal pull-up enabled

  // Set up TimeSet pins
  pinMode(TimeSetButton, INPUT);    // declare pushbutton as input
  digitalWrite(TimeSetButton, HIGH); // internal pull-up enabled
  delay(3000); // Delay start GPS
  
  // Set GPS input default to 9600 baud
  Serial.begin(9600);
  Serial.println("$PSIMIPR,W,4800*17"); // Init GPS NMEA speed 4800 (CRC Calculator http://www.hhhh.org/wiml/proj/nmeaxor.html)
  delay(500);
  Serial.begin(4800);
  Serial.println("$PMTK314,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0*28"); // Init GPS NMEA output
  delay(500);
    
   // Initialize a buffer for received data
  for (int i = 0; i < 300; i++) {
    buffer[i] = ' ';
  }
    
  pinMode(HourSetButton, INPUT);     // declare pushbutton as input
  digitalWrite(HourSetButton, HIGH); // internal pull-up enabled
  pinMode(MinSetButton, INPUT);   // declare pushbutton as input
  digitalWrite(MinSetButton, HIGH); // internal pull-up enabled

  // Set up DDS-60
  pinMode (data, OUTPUT);   // sets pin 38 as OUPUT
  pinMode (clock, OUTPUT);  // sets pin 37 as OUTPUT
  pinMode (load, OUTPUT);   // sets pin 36 as OUTPUT


  // turn off transmitter
  TempWord = 0;
  TransmitSymbol();

  // Load Cal Factor
  CalFactor.b1 = EEPROM.read(50);
  CalFactor.b2 = EEPROM.read(51);
  CalFactor.b3 = EEPROM.read(52);
  CalFactor.b4 = EEPROM.read(53);

  setfreq();

  // Begin WSPR message calculation
  encode_call();
  encode_locator();
  encode_conv();
  interleave_sync();


  // Calibration process follows:
  if (digitalRead(CalSetButton) == LOW)
  {
    FreqWord = TenMHz * pow(2, 32) / fCLK;
    TempWord = FreqWord;
    TransmitSymbol();
    CalFactor.value = 0;
    detachInterrupt(0);  // Disable transmit inhibit interrupt
    TIMSK1 = 0;          // Disable timer1 interrupt (master clock)
    TIMSK2 = 0;          // Disable timer2 interrupt (WSPR)

    lcd.setCursor(0, 0);
    lcd.print("Adjust to 10MHz");
    lcd.setCursor(0, 1);
    lcd.print("Cal Factor= ");
    lcd.setCursor(12, 1);
    lcd.print(CalFactor.value);
    calibrate();
    lcd.setCursor(0, 0);
    lcd.print("                ");
    lcd.setCursor(0, 1);
    lcd.print("                ");
    setfreq();
  }

  // Manual time set process follows:
  if (digitalRead(TimeSetButton) == LOW)
  {
    TIMSK1 = 0;        // Interrupt disable to stop clock
    hour = 0;
    minute = 0;
    second = 0;
    displaytime();
    detachInterrupt(0);  // Disable transmit inhibit interrupt
    timeset();
  }
}

//******************************************************************

void loop()
{
  if (RXflag == 1)
  {
    GPSprocess();
  }
  if (flagUpdateTime) {
      flagUpdateTime = 0;
      displaytime();
  }
  if (flagUpdateTx) {
      flagUpdateTx = 0;
      if (GPSinhibitFlag != 1 && InhibitFlag != 1) {
          setfreq();
          transmit();
      }
  }
}

//******************************************************************
void encode()
{
  encode_call();
  encode_locator();
  encode_conv();
  interleave_sync();
};
//******************************************************************
// normalize characters 0..9 A..Z Space in order 0..36
char chr_normf(char bc )
{
  char cc = 36;
  if (bc >= '0' && bc <= '9') cc = bc - '0';
  if (bc >= 'A' && bc <= 'Z') cc = bc - 'A' + 10;
  if (bc >= 'a' && bc <= 'z') cc = bc - 'a' + 10;
  if (bc == ' ' ) cc = 36;

  return (cc);
}

//******************************************************************
void encode_call()
{
  unsigned long t1;

  // coding of callsign
  if (chr_normf(call[2]) > 9)
  {
    call[5] = call[4];
    call[4] = call[3];
    call[3] = call[2];
    call[2] = call[1];
    call[1] = call[0];
    call[0] = ' ';
  }

  n1 = chr_normf(call[0]);
  n1 = n1 * 36 + chr_normf(call[1]);
  n1 = n1 * 10 + chr_normf(call[2]);
  n1 = n1 * 27 + chr_normf(call[3]) - 10;
  n1 = n1 * 27 + chr_normf(call[4]) - 10;
  n1 = n1 * 27 + chr_normf(call[5]) - 10;

  // merge coded callsign into message array c[]
  t1 = n1;
  c[0] = t1 >> 20;
  t1 = n1;
  c[1] = t1 >> 12;
  t1 = n1;
  c[2] = t1 >> 4;
  t1 = n1;
  c[3] = t1 << 4;
}

//******************************************************************
void encode_locator()
{
  unsigned long t1;
  // coding of locator
  m1 = 179 - 10 * (chr_normf(locator[0]) - 10) - chr_normf(locator[2]);
  m1 = m1 * 180 + 10 * (chr_normf(locator[1]) - 10) + chr_normf(locator[3]);
  m1 = m1 * 128 + power + 64;

  // merge coded locator and power into message array c[]
  t1 = m1;
  c[3] = c[3] + ( 0x0f & t1 >> 18);
  t1 = m1;
  c[4] = t1 >> 10;
  t1 = m1;
  c[5] = t1 >> 2;
  t1 = m1;
  c[6] = t1 << 6;
}

//******************************************************************
// convolutional encoding of message array c[] into a 162 bit stream
void encode_conv()
{
  int bc = 0;
  int cnt = 0;
  int cc;
  unsigned long sh1 = 0;

  cc = c[0];

  for (int i = 0; i < 81; i++) {
    if (i % 8 == 0 ) {
      cc = c[bc];
      bc++;
    }
    if (cc & 0x80) sh1 = sh1 | 1;

    symt[cnt++] = parity(sh1 & 0xF2D05351);
    symt[cnt++] = parity(sh1 & 0xE4613C47);

    cc = cc << 1;
    sh1 = sh1 << 1;
  }
}

//******************************************************************
byte parity(unsigned long li)
{
  byte po = 0;
  while (li != 0)
  {
    po++;
    li &= (li - 1);
  }
  return (po & 1);
}

//******************************************************************
// interleave reorder the 162 data bits and and merge table with the sync vector
void interleave_sync()
{
  int ii, ij, b2, bis, ip;
  ip = 0;

  for (ii = 0; ii <= 255; ii++) {
    bis = 1;
    ij = 0;
    for (b2 = 0; b2 < 8 ; b2++) {
      if (ii & bis) ij = ij | (0x80 >> b2);
      bis = bis << 1;
    }
    if (ij < 162 ) {
      sym[ij] = SyncVec[ij] + 2 * symt[ip];
      ip++;
    }
  }
}

//******************************************************************
// Determine time slot and load band frequency data. Display
// frequency and calculate frequency word for DDS
void setfreq()
{
  // Select band port pin and set HIGH for active band
  if (bitRead(timeslot, 0) == 1)digitalWrite(BandBCD0pin, HIGH);
  else
    digitalWrite(BandBCD0pin, LOW);
  if (bitRead(timeslot, 1) == 1)digitalWrite(BandBCD1pin, HIGH);
  else
    digitalWrite(BandBCD1pin, LOW);
  if (bitRead(timeslot, 2) == 1)digitalWrite(BandBCD2pin, HIGH);
  else
    digitalWrite(BandBCD2pin, LOW);
  if (bitRead(timeslot, 3) == 1)digitalWrite(BandBCD3pin, HIGH);
  else
    digitalWrite(BandBCD3pin, LOW);

  // Print frequency to the LCD
  lcd.setCursor(0, 0);

  ltoa(band[timeslot], buf, 10);

  if (buf[7] == 0) {
    lcd.print(buf[0]);
    lcd.print(' ');
    lcd.print(buf[1]);
    lcd.print(buf[2]);
    lcd.print(buf[3]);
    lcd.print(' ');
    lcd.print(buf[4]);
    lcd.print(buf[5]);
    lcd.print(buf[6]);
    lcd.print(" Hz ");
  }
  else {
    lcd.print(buf[0]);
    lcd.print(buf[1]);
    lcd.print(' ');
    lcd.print(buf[2]);
    lcd.print(buf[3]);
    lcd.print(buf[4]);
    lcd.print(' ');
    lcd.print(buf[5]);
    lcd.print(buf[6]);
    lcd.print(buf[7]);
    lcd.print(" Hz");
  }

  FreqWord = (band[timeslot] + (CalFactor.value * (band[timeslot] / pow(10, 7)))) * pow(2, 32) / fCLK;

  return;
}


//******************************************************************
// Determine if it is time to transmit. If so, determine if it is
// time to transmit the QRSS or the WSPR message

void transmit()
{
  if (TransmitFlag[timeslot] == 1) { // Start WSPR transmit process
    start = 0;
    RXflag = 0;     // Disable GPS receiver
    lcd.setCursor(9, 1);
    lcd.print("WSPR TX");
    lcd.setCursor(14, 0);
    lcd.print("  ");
    digitalWrite(txPin, HIGH); // External transmit control ON
    bb = 0;
    count = 0;            // Start WSPR symbol transmit process
    TIMSK2 = 2;   //Enable timer2 interrupt
    return;
  } else { // Turn off transmitter and idle
    TIMSK2 = 0; //Turn off WSPR timer
    start = 0;
    GpsEnable(); //Enable GPS receiver
    lcd.setCursor(9, 1);
    lcd.print("IDLE   ");
    digitalWrite(txPin, LOW); // External transmit control OFF
    TempWord = 0;
    TransmitSymbol();
    return;
  }
}

//******************************************************************
void TransmitSymbol()
{
  for (mask = 1; mask > 0; mask <<= 1) { //iterate through bit mask
    if (TempWord & mask) { // if bitwise AND resolves to true
      digitalWrite(data, HIGH); // send 1
    }
    else { //if bitwise and resolves to false
      digitalWrite(data, LOW); // send 0
    }
    digitalWrite(clock, HIGH); // Clock data in
    digitalWrite(clock, LOW);
  }
  for (temp = 1; temp > 0; temp <<= 1) { //iterate through bit mask
    if (w0 & temp) { // if bitwise AND resolves to true
      digitalWrite(data, HIGH); // send 1
    }
    else { //if bitwise and resolves to false
      digitalWrite(data, LOW); // send 0
    }
    digitalWrite(clock, HIGH); // Clock data in
    digitalWrite(clock, LOW);
  }
  digitalWrite (load, HIGH); // Turn on pin 36/1 LOAD FQ-UD
  digitalWrite (load, LOW); // Turn pin 36 off
  return;
}

/*
  _______________________________________________________________
  GPS timing process starts here
  _______________________________________________________________
*/
void GPSprocess()
{
  byte pinState = 0;
  byte gpserr = 0;
  byte flagGpsChange = 0;
  byteGPS = Serial.read();       // Read a byte of the serial port
  if (counter >= sizeof(buffer)) {
      counter = 0;
  }
  if (byteGPS == -1) {           // See if the port is empty yet

  } else {
    buffer[counter] = byteGPS;      // If there is serial port data, it is put in the buffer
    counter++;
    if (byteGPS == 13) {         // If the received byte is = to 13, end of transmission
      IndiceCount = 0;
      StartCount = 0;
      for (int i = 1; i < 7; i++) { // Verify the received command starts with $GPGGA
        if (buffer[i] == StartCommand[i - 1]) {
          StartCount++;
        }
      }
      validGPSflag = 0;
      if (StartCount == 6) {           // If yes, continue and process the data
          flagGpsChange = 1;
        for (int i = 0; i < 300; i++) {
          if (buffer[i] == ',') { // check for the position of the  "," separator
            indices[IndiceCount] = i;
            IndiceCount++;
          }
          if (IndiceCount >= IndicesCount || IndiceCount > 7) {
              break;
          }
        }
        if (IndiceCount > 7) {
          // Load time data
          temp = indices[0];
          hour = ((buffer[temp + 1] - 48) * 10 + (buffer[temp + 2] - 48) + 3) % 24; // UTC+3
          minute = (buffer[temp + 3] - 48) * 10 + buffer[temp + 4] - 48;
          second = (buffer[temp + 5] - 48) * 10 + buffer[temp + 6] - 48;
          // Load latitude and logitude data
          temp = indices[1];
          Lat10 = buffer[temp + 1] - 48;
          Lat1 = buffer[temp + 2] - 48;
          temp = indices[2];
          NS = buffer[temp + 1];
          temp = indices[3];
          Lon100 = buffer[temp + 1] - 48;
          Lon10 = buffer[temp + 2] - 48;
          Lon1 = buffer[temp + 3] - 48;
          temp = indices[4];
          EW = buffer[temp + 1];
          temp = indices[5];
          validGPSflag = buffer[temp + 1] - 48;
          temp = indices[6];
          byte delta = indices[7] - temp;
          if (delta > 2) {
            sat10 = buffer[temp + 1] - 48;
            sat1 = buffer[temp + 2] - 48;
          } else {
              if (delta > 1) {
                  sat10 = 0;
                  sat1 = buffer[temp + 1] - 48;
              } else {
                  gpserr = 1;
              }
          }
        }
        if (validGPSflag > 0)GPSinhibitFlag = 0;
        else
        {
          GPSinhibitFlag = 1;
        }
      }
      counter = 0;                  // Reset the buffer
      for (int i = 0; i < 300; i++) { //
        buffer[i] = ' ';
      }

    }
  }
  if (flagGpsChange) {
      flagGpsChange = 0;
      lcd.setCursor(14, 0);
      if (gpserr) {
          lcd.print("--");
      } else {
          lcd.print(sat10);
          lcd.print(sat1);
      }
      lcd.setCursor(9, 1);
      if (GPSinhibitFlag == 0) {
          lcd.print("IDLE   ");
      } else {
          lcd.print(" NO GPS");
      }
  }
}

//******************************************************************
void calibrate()
{ // Process to determine frequency calibration factor
  while (digitalRead(CalSetButton) == LOW)
  {
    if (digitalRead(CalUpButton) == LOW)
    {
      for (TempWord = 0; TempWord < 350000; TempWord++) {
      }; //crude debounce delay
      CalFactor.value++;
      delay(100);
    };
    if (digitalRead(CalDwnButton) == LOW)
    {
      for (TempWord = 0; TempWord < 350000; TempWord++) {
      }; //crude debounce delay
      CalFactor.value--;
      delay(100);
    };
    TempWord = (TenMHz + CalFactor.value) * pow(2, 32) / fCLK;
    TransmitSymbol();
    lcd.setCursor(12, 1);
    lcd.print(CalFactor.value);
    lcd.print("   ");
  }


  // Writes CalFactor to address 50 + 3 bytes of EEprom
  EEPROM.write(50, CalFactor.b1);
  EEPROM.write(51, CalFactor.b2);
  EEPROM.write(52, CalFactor.b3);
  EEPROM.write(53, CalFactor.b4);

  lcd.setCursor(9, 1);
  lcd.print("IDLE   ");
  TempWord = 0; // turn off 10 MHz calibrate signal
  TransmitSymbol();
  setfreq();
  TIMSK1 = 2;          // Enable Timer1 Interrupt
}


//******************************************************************
void timeset()
{ // Process to manually set master time clock
  lcd.setCursor(9, 1);
  lcd.print("SetTime");
  while (digitalRead(TimeSetButton) == LOW)
  {
    if (digitalRead(MinSetButton) == LOW)
    {
      for (TempWord = 0; TempWord < 750000; TempWord++) {
      }; //crude debounce delay
      minute++;
      if (minute > 59) minute = 0;
      displaytime();
    };
    if (digitalRead(HourSetButton) == LOW)
    {
      for (TempWord = 0; TempWord < 750000; TempWord++) {
      }; //crude debounce delay
      hour++;
      if (hour > 23) hour = 0;
      displaytime();
    };
  }

  TempWord = 0;
  TransmitSymbol();
  lcd.setCursor(9, 1);
  lcd.print("IDLE   ");
  TIMSK1 = 2;          // Enable Timer1 Interrupt
}


//******************************************************************
void displaytime()
{
  lcd.setCursor(0, 1);
  lcd.display();
  if (hour < 10) lcd.print ("0");
  lcd.print (hour);
  lcd.print (":");
  if (minute < 10) lcd.print ("0");
  lcd.print (minute);
  lcd.print (":");
  if (second < 10) lcd.print ("0");
  lcd.print (second);
  lcd.print (" ");
  return;
}
